<?php

/**
 * @author Zarif Safiullin <zaph.work@gmail.com>
 * @package ow.ow_plugins.mailbox.bol
 * @since 1.6.1
 */
class MAILBOX_BOL_AjaxService {

    const MAX_MESSAGE_TEXT_LENGTH = 24000;
    /**
     * Class instance
     *
     * @var MAILBOX_BOL_Service
     */
    private static $classInstance;

    /**
     * Returns class instance
     *
     * @return MAILBOX_BOL_Service
     */
    public static function getInstance() {
        if (self::$classInstance === null) {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * Class constructor
     *
     */
    protected function __construct() {

    }

    public function postMessage($params)
    {
        $conversationService = MAILBOX_BOL_ConversationService::getInstance();
        $language = OW::getLanguage();

        if ($errorMessage = $conversationService->checkPermissions())
        {
            return array('error'=>$errorMessage);
        }

        $userId = OW::getUser()->getId();
        $conversationId = $params['convId'];
        if ( !isset($conversationId) )
        {
            return array('error'=>"Conversation is not defined");
        }

        if ( empty($params['text']) )
        {
            return array('error'=>$language->text('mailbox', 'chat_message_empty'));
        }

        if (mb_strlen($params['text']) > self::MAX_MESSAGE_TEXT_LENGTH)
        {
            return array( 'error'=>$language->text('mailbox', 'message_too_long_error', array('maxLength' => self::MAX_MESSAGE_TEXT_LENGTH)) );
        }

        $conversation = $conversationService->getConversation($conversationId);
        $opponentId = $conversation->initiatorId == $userId ? $conversation->interlocutorId : $conversation->initiatorId;

        $checkResult = $conversationService->checkUser($userId, $opponentId);

        if ( $checkResult['isSuspended'] )
        {
            return array('error'=>$checkResult['suspendReasonMessage']);
        }
        $mode = $conversationService->getConversationMode($conversationId);

        $actionName = '';

        switch($mode)
        {
            case 'chat':

                $firstMessage = $conversationService->getFirstMessage($conversationId);

                if (empty($firstMessage))
                {
                    $actionName = 'send_chat_message';
                }
                else
                {
                    $actionName = 'reply_to_chat_message';
                }

                $isAuthorized = OW::getUser()->isAuthorized('mailbox', $actionName);
                if ( !$isAuthorized )
                {
                    $status = BOL_AuthorizationService::getInstance()->getActionStatus('mailbox', $actionName);
                    if ( $status['status'] != BOL_AuthorizationService::STATUS_AVAILABLE )
                    {
//                        return array('error'=>$language->text('mailbox', $actionName.'_permission_denied'));
                        return array('error'=>$status['msg']);
                    }
                }
                break;

            case 'mail':
                $actionName = 'reply_to_message';
                $isAuthorized = OW::getUser()->isAuthorized('mailbox', $actionName);
                if ( !$isAuthorized )
                {
                    $status = BOL_AuthorizationService::getInstance()->getActionStatus('mailbox', $actionName);
                    if ( $status['status'] != BOL_AuthorizationService::STATUS_AVAILABLE )
                    {
//                        return array('error'=>$language->text('mailbox', $actionName.'_permission_denied'));
                        return array('error'=>$status['msg']);
                    }
                }
                break;
        }

        $params['text'] = UTIL_HtmlTag::stripTags(UTIL_HtmlTag::stripJs($params['text']));

        $event = new OW_Event('mailbox.before_send_message', array(
            'senderId' => $userId,
            'recipientId' => $opponentId,
            'conversationId' => $conversation->id,
            'message' => $params['text']
        ), array('result' => true, 'error' => '', 'message' => $params['text'] ));
        OW::getEventManager()->trigger($event);

        $data = $event->getData();

        if ( !$data['result'] )
        {
            return $data;
        }

        $text = $data['message'];

        try
        {
            $message = $conversationService->createMessage($conversation, $userId, $text);

            $files = BOL_AttachmentService::getInstance()->getFilesByBundleName('mailbox', $params['uid']);

            if (!empty($files))
            {
                $conversationService->addMessageAttachments($message->id, $files);
            }

            if (!empty($params['embedAttachments']))
            {
                $oembedParams = json_decode($params['embedAttachments'], true);
                $oembedParams['message'] = $text;
                $messageParams = array(
                    'entityType'=>'mailbox',
                    'eventName'=>'renderOembed',
                    'params'=>$oembedParams
                );
                $message->isSystem = true;
                $message->text = json_encode($messageParams);

                $conversationService->saveMessage($message);
            }
        }
        catch(InvalidArgumentException $e)
        {
            return array('error'=>$e->getMessage());
        }

        if (!empty($actionName))
        {
            BOL_AuthorizationService::getInstance()->trackAction('mailbox', $actionName);
        }

        $item = $conversationService->getMessageData($message);

        return array('message'=>$item);
    }

    public function getLog($params)
    {
        $conversationService = MAILBOX_BOL_ConversationService::getInstance();

        if ($errorMessage = $conversationService->checkPermissions())
        {
            return array('error'=>$errorMessage);
        }

        if (!OW::getUser()->isAuthenticated())
        {
            return array();
        }

        $userId = OW::getUser()->getId();
        $conversationId = (int)$params['convId'];

        $opponentId = (int)$params['opponentId'];
        if (!empty($opponentId))
        {
            if ( empty($conversationId) )
            {
                $conversationId = $conversationService->getChatConversationIdWithUserById($userId, $opponentId);
                if ( empty($conversationId) )
                {
                    $conversation = $conversationService->createChatConversation($userId, $opponentId);

                    $conversationId = $conversation->getId();
                }
            }
        }

        if (!empty($params['markRead']))
        {
            $conversationService->markRead(array($conversationId), $userId);
            $conversationService->setConversationViewedInConsole(array($conversationId), $userId);
        }

        return $conversationService->getConversationDataAndLog($conversationId);
    }

    /**
     * Marks conversation as UnRead
     *
     * @param array $params
     * @return boolean
     */
    public function markConversationUnRead( $params )
    {
        $userId = OW::getUser()->getId();
        $language = OW::getLanguage();

        if ( !OW::getUser()->isAuthenticated() || $userId === null )
        {
            return array('error'=>'User is not authenticated');
        }

        if ( empty($params['conversationId']) )
        {
            return array('error'=>'Mark conversation as unread fail! \nEmpty param conversationId!');
        }

        $conversation = (int) $params['conversationId'];
        $conversationService = MAILBOX_BOL_ConversationService::getInstance();

        try
        {
            $conversationService->markUnRead(array($conversation), $userId);
        }
        catch ( Exception $e )
        {
            return array('error' => $language->text('mailbox', 'mark_unread_fail_message'));
        }

        return array( 'result'=>true, 'notice'=>$language->text('mailbox', 'mark_conversation_unread_message'));
    }

    /**
     * Marks conversation as Read
     *
     * @param array $params
     * @return boolean
     */
    public function markConversationRead( $params )
    {
        $userId = OW::getUser()->getId();
        $language = OW::getLanguage();

        if ( !OW::getUser()->isAuthenticated() || $userId === null )
        {
            return array('error'=>'User is not authenticated');
        }

        if ( empty($params['conversationId']) )
        {
            return array('error'=>'Mark conversation as read fail! \nEmpty param conversationId!');
        }

        $conversationId = (int) $params['conversationId'];
        $conversationService = MAILBOX_BOL_ConversationService::getInstance();

        try
        {
            $conversationService->markRead(array($conversationId), $userId);
        }
        catch ( Exception $e )
        {
            return array('error' => $language->text('mailbox', 'mark_read_fail_message'));
        }

        return array( 'result'=>true );
    }
}